;;; Programa: Interfaz para la creacion de sistemas de 
;;;           produccion simples
;;; Autor: Vctor Rodriguez Herreros
;;; Fecha: 29 de Septiembre de 1997
;;; Comentarios:
;;;    La finalidad de este programa es la de crear una 
;;;    herramienta en Lisp que permita la creacion de 
;;;    sistemas de produccion para la resolucion de 
;;;    diferentes problemas. Por lo tanto, esta interfaz 
;;;    es independiente del tipo de problema y no hace 
;;;    ninguna suposicion respecto al tipo de problema
;;;
;;; Historia:
;;;   2 de Octubre de 1997 - Terminada la implementacion de 
;;;                          las funciones REGLA e HIJOS
;;;   Mayo 1998 - Modificaciones de maquillaje (Patricio)

;;; ****************************************************
;;; Documentacion **************************************
;;; ****************************************************
;;;
;;; SHELL - Interfaz para la implementacion de sistemas 
;;;         de produccion
;;; Programado originalmente con CLISP para OS/2
;;; Adaptado para su uso generico en ANSI COMMON LISP
;;; 
;;; Uso de la liberia:
;;; 
;;; (REGLA PRECONDICION ACCION)
;;; 
;;; Esta funcion se utiliza para introducir reglas en el 
;;; sistema de produccion. El primer parmetro es la funion
;;; booleana que evalua una precondicion para un estado 
;;; dado. El segundo parmetro se corresponde con la funcio 
;;; que cre el estado correspondiente a la accion aplicada 
;;; sobre un estado cuando en este se cumpla la precondicion 
;;; evualuada por el primer parmetro de la funcio.
;;; Por tanto tanto las funciones PRECONDICION como ACCION 
;;; reciben como parametros un estado y devuelven T o NIL y 
;;; el nuevo estado respectivamente.
;;; 
;;; (HIJOS PADRE)
;;; 
;;; Esta funcion se utiliza para obtener los nuevos estados 
;;; a los que se puede llegar mediante la aplicacion de las 
;;; reglas del sistema de produccion creado previamente 
;;; utilizando la funcion REGLA. Se retorna una lista con
;;; todos los estados a los que se ha podido evolucionar 
;;; desde el estado inicial pasado como parmetro
;;; 
;;; *SISTEMA*
;;; 
;;; Esta variable es de uso interno de la libreria y no 
;;; deberia ser modificada directamente por el usuario. 
;;; En ella se almacena el sistema de produccion creado 
;;; con la funcion REGLA.
;;; 

;;; La siguiente variable almacena todas las reglas que 
;;; se han creado para el sistema de produccion. 
;;; Se inicializa con el valor lista vacia = NIL

(defvar *sistema* nil
  "Esta variable almacena en una lista todas las reglas que se han creado para el sistema de produccion.")

;;; La funcion REGLA se encarga de aadir una nueva regla 
;;; al sistema de produccion. Se pasan dos parmetros, la
;;; precondicion de la misma y la accion a ejecutar en 
;;; caso de que la precondicion se cumpla
(defun regla (precondicion accion)
  "
    - Funcion: REGLA (PRECONDICION ACCION)

        Funcion que se encarga de aadir una nueva regla al sistema de produccion (almacenado en la variable *sistema*). Se le pasan dos parmetros, la precondicion de la misma y la accion a ejecutar en caso de que la precondicion se cumpla"
  ; Se comprueba si es la primera regla a aadir
  (push (list precondicion 
	      accion )
	*sistema*))
; Final del DEFUN REGLA

;;; La funcion HIJOS devuelve una lista con todos los 
;;; estados a los que se puede evolucionar siempre y 
;;; cuando se cumpla la precondicion de la regla 
;;; correspondiente. El estado desde el que se genera
;;; esta lista de estados se pasar como parmetro a  
;;; funcion
(defun hijos (padre)
  "Funcion que devuelve una lista con todos los estados a los que se puede evolucionar siempre y cuando se cumpla la precondicion de la regla correspondiente. El estado desde el que se genera esta lista de estados se pasar como parmetro a la funcin"
  ; La variable local LISTA almacenara a los estados 
  ; generados por la aplicacion de las reglas de nuestro 
  ; sistema de produccion
  (let ((lista nil))
    ; Para cada regla del sistema de produccion ...
    (dolist (patron *sistema* lista)
      ; Si se cumple la precondicion ...
      (when (funcall (first patron) padre)
      ; Ejecutamos la precondicion. Hay que aadir el 
      ; nuevo estado a la lista de hijos generados
	(push (funcall (first (rest patron)) padre) lista)))))


